##
## Module to fetch_stat.pl from the Dell PowerConnect 2748
## Has only been tested on FW 1.0.0.33
##
## Written by Joakim Andersson, joakim at iqcc dot se
## Thanks to Jason Keltz for the access to your switch
##

use strict;
use LWP::UserAgent;
use Net::HTTP;

our $modver  = '1.00b (2010-03-09)';
our $modname = "dell_28xx.pm";
our $modcreator = "Joakim Andersson";

#######################################################
# Shared variables

our $SET;			# Settings
our $SYSTEM;		# System-info
our $loop;			# Current loop

# Only these names should be considered as variables in the RMON-page
# Map to the old names used in 2708/16/24
my %rmon_name_map = (
	'Received Bytes (Octets)' => 'goodOctetRCV',
	'Received Packets' => 'rxGood',
	'Broadcast Packets Received' => 'rxBroadcast',
	'Multicast Packets Received' => 'rxMulticast',
	'CRC&Align Errors' => 'rxFCS',
	'Undersize Packets' => 'rxUnderSize',
	'Oversize Packets' => 'rxOverSize',
	'Fragments' => 'rxFragment',
	'Jabbers' => 'rxJabber',
	'Collisions' => 'txCollisions',
	'Frames of 64 Bytes' => 'rx64Octets',
	'Frames of 65 to 127 Bytes' => 'rx65TO127Octets',
	'Frames of 128 to 255 Bytes' => 'rx128TO255Octets',
	'Frames of 256 to 511 Bytes' => 'rx256TO511Octets',
	'Frames of 512 to 1023 Bytes' => 'rx512TO1023Octets',
	'Frames of 1024 to 1632 Bytes' => 'rx1024ToMa',
);

# Creating UserAgent-object
my $ua = LWP::UserAgent->new();
$ua->agent("Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)");

##
## Poll port
##

sub switch_poll_port
{
	my $portnum = shift;
	my $is_first_poll = shift || 0;
	
	my $logtag = "$modname: poll_port:";
	
	##
	## Is it a port or LAG?
	##
	
	$portnum =~ /^(T)?(\d+)$/ || &log_n_die("$logtag Invalid portnum $portnum!");
	
	my $is_trunk = ($1) ? 1 : 0;
	my $arg = ($is_trunk) ? $SYSTEM->{'numberOfPorts'}+4+$2 : $portnum;
	
	# Fetches the page
	my $res = $ua->get("http://$SET->{'SWITCH'}/config/rmon_stats.htm?Query:etherStatsIndex=$arg",
	       'Cookie' => "$SYSTEM->{'cookie'}; checked=true;"
		 );
	
	# Parsing content
	my $data = $res->content;
	$data =~ tr/\r//d;
	
	if ($SET->{'DEBUG'})
	{
		open (DUMP, ">$SYSTEM->{'BASEDIR'}/debug_config-rmon_stats.htm");
		print DUMP $data;
		close DUMP;
	}
	
	if ($data =~ /^<title>Authentication Page<\/title>$/m)
	{
		##
		## Invalid session!
		##
		
		&log("Session invalid", $SET->{'VERBOSE'});
		
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	##
	## Parses the data, is stored in javascript-variables in the beginning of the page
	##
	
	my $row = 0;
	
	my $name = '';
	my %vars = ();
	
	foreach (split(/\n+/, $data))
	{
		$row++;
		#print "RAD $row: $_\n";
		#<td nowrap width=50% class="data-area-canvas-odd">Received Bytes (Octets)</td>
		#<td nowrap width=50% class="data-area-canvas-odd">
		#2268456512
		#&nbsp;
		#</td>
		
		if (/^<td nowrap .*>(.+)<\/td>$/)
		{
			#print "$name\n";
			$name = $rmon_name_map{$1} || '';
			next;
		}
		next unless ($name);
		next unless (/^(\d+)$/ || /NAME=etherStatsCollisions.*value="(\d+)"/i);
		
		my $value = $1;
		$value = '' if ($is_trunk && $name eq 'txCollisions');
		
		$vars{$name} = $value;
	}
	
	return \%vars;
}

##
## Resets stats for all ports
## Not implemented for the 28xx, seems to be too much headache
##

sub switch_clear_stat
{
	my $logtag = "$modname: clear_stat:";
	return 0;
}

##
## Reading systeminfo that doesn't change (often)
##

sub switch_system_info
{
	my $logtag = "$modname: system_info:";
	
	#?[ifTableVT]Filter:((ifIndex>=1)&&(ifIndex<=56))
	my $res = $ua->get("http://$SET->{'SWITCH'}/config/port_config_showall.htm",
	       'Cookie' => "$SYSTEM->{'cookie'}; checked=true;"
		 );
	
	&log_n_die("$logtag Unable to fetch /config/port_config_showall.htm - ".$res->status_line) unless ($res->is_success);
	
	# Content
	my $data = $res->content;
	$data =~ tr/\r//d;
	
	if ($SET->{'DEBUG'})
	{
		open (DUMP, ">$SYSTEM->{'BASEDIR'}/debug_config-port_config_show_all.htm");
		print DUMP $data;
		close DUMP;
	}
	
	if ($data =~ /^<title>Authentication Page<\/title>$/m)
	{
		##
		## Invalid session!
		##
		
		&log("Session invalid", $SET->{'VERBOSE'});
		
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	##
	## Finds the info we needs
	##

	#<INPUT  type="hidden" name=ifIndex$repeat?2    value="2" SIZE="20" MAXLENGTH="20">
	
	my $row = 0;
	my %vars = ();
	my $tot = 0;	# 
	
	foreach (split(/\n+/, $data))
	{
		$row++;
		#print "RAD $row: $_\n";
		
		next unless (/^<input.*name=\"?([^\" ]+)\"?.*value=\"?([^\" ]+)\"?/i);
		
		my $name = $1;
		my $value = $2;
		
		#next if ($stat{$name});
		#print "  $name -> $value\n";
		$vars{$name} = $value;
		
		# Highest ifIndex-number
		$tot = $1 if ($name =~ /^ifIndex\$repeat\?(\d+)$/);
	}
	
	##
	## Parses info about every port
	##
	
	my %ports = ();
	my $order = 0;
	my $numberOfPorts = 0;
	my $numberOfTrunks = 0;
	
	foreach my $num (1 .. $tot)
	{
		#<INPUT  type="hidden" name=ifName$repeat?2    value="g2" SIZE="20" MAXLENGTH="20">
		my $ifName = $vars{"ifName\$repeat?$num"};
		
		my $portnum = '';
		if ($ifName =~ /^g(\d+)$/)
		{
			# Normal gigport
			$numberOfPorts = $portnum = $1;
			#print "port $numberOfPorts\n";
		}
		elsif ($ifName =~ /^ch(\d+)$/)
		{
			# LAG, I keep the T...
			$portnum = "T$1";
			$numberOfTrunks = $1;
			#print "trunk $numberOfTrunks\n";
		}
		else
		{
			last;
		}
		
		#<INPUT  type="hidden" name=ifOperStatus$repeat?2    value="2" SIZE="20" MAXLENGTH="20">
		my $ifOperStatus = $vars{"ifOperStatus\$repeat?$num"};
		next if ($ifOperStatus == 6);	# Disabled, probably not created LAG?
		
		#<INPUT  type="hidden" name=ifHighSpeed$repeat?2    value="1000" SIZE="20" MAXLENGTH="20">
		#<INPUT  type="hidden" name=swIfDuplexOperMode$repeat?2    value="4" SIZE="20" MAXLENGTH="20">
		#<INPUT  type="hidden" name=swIfOperBackPressureMode$repeat?2    value="4" SIZE="20" MAXLENGTH="20">
		#<INPUT  type="hidden" name=swIfOperFlowControlMode$repeat?2    value="2" SIZE="20" MAXLENGTH="20">
		my $portSpeed	= $vars{"ifHighSpeed\$repeat?$num"};			# 10 100 1000
		my $portDuplex	= $vars{"swIfDuplexOperMode\$repeat?$num"};
		my $portBP		= $vars{"swIfOperBackPressureMode\$repeat?$num"};
		my $portFc		= $vars{"swIfOperFlowControlMode\$repeat?$num"};
		
		my $link = 'down';	# ifOperStatus == 2
		
		if ($ifOperStatus == 1)
		{
			$link = $portSpeed;
			#$link = 'active' if ($portSpeed == 3);
			$link .= '/Half' if ($portDuplex == 1);
			$link .= '/Full' if ($portDuplex == 2);
			$link .= '/FC' if ($portFc == 1);
			$link .= '/rxFC' if ($portFc == 3);
			$link .= '/txFC' if ($portFc == 4);
			$link .= '/BP' if ($portBP == 1);
		}
		
		#print "$link\n";
		
		$ports{$portnum} = {};
		$ports{$portnum}->{'link'} = $link;
		$ports{$portnum}->{'is_active'} = ($ifOperStatus == 1);
		$ports{$portnum}->{'order'} = $num;
	}
	
	##
	## All ports are parsed
	##
	
	$SYSTEM->{'numberOfPorts'} = $numberOfPorts;
	$SYSTEM->{'numberOfTrunks'} = $numberOfTrunks;
	
	##
	## Fetching system info (asset)
	##
	
	$res = $ua->get("http://$SET->{'SWITCH'}/config/general_asset.htm",
	       'Cookie' => "$SYSTEM->{'cookie'}; checked=true;"
		 );
	
	&log_n_die("$logtag Unable to fetch /config/general_asset.htm - ".$res->status_line) unless ($res->is_success);
	
	# Parsing content
	$data = $res->content;
	$data =~ tr/\r//d;
	
	if ($SET->{'DEBUG'})
	{
		open (DUMP, ">$SYSTEM->{'BASEDIR'}/debug_config-general_asset.htm");
		print DUMP $data;
		close DUMP;
	}
	
	if ($data =~ /^<title>Authentication Page<\/title>$/m)
	{
		##
		## Invalid session!
		##
		
		&log("Session invalid", $SET->{'VERBOSE'});
		
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	##
	## Parses the data, is stored in javascript-variables in the beginning of the page
	##
	
	#<INPUT  TYPE=hidden NAME=productIdentificationServiceTag$repeat?1    value="25TQFH1" SIZE="20" MAXLENGTH="20">
	#<INPUT  TYPE=hidden NAME=productIdentificationAssetTag$repeat?1    value="&nbsp;" SIZE="20" MAXLENGTH="20">
	#<INPUT  TYPE=hidden NAME=productIdentificationSerialNumber$repeat?1    value="CN0F491K282980171375A01" SIZE="20" MAXLENGTH="20">
	#<INPUT  TYPE=text NAME="sysName" size="35" maxlength="160"   value="switch2824attic">
	#<INPUT  TYPE=text NAME="sysContact" size="35" maxlength="160"   value="Mike&nbsp;Trogni&nbsp;203-637-1232">
	#<INPUT  TYPE=text NAME="sysLocation" size="35" maxlength="160"   value="10&nbsp;Shady&nbsp;Brook&nbsp;Ln,&nbsp;Attic">
	#<INPUT  TYPE=hidden NAME="rndBasePhysicalAddress"   value="a4badb5a44f6" SIZE="20" MAXLENGTH="20">
	#<INPUT  TYPE=hidden NAME="sysUpTime"   value="91088500" SIZE="20" MAXLENGTH="20">
	
	$row = 0;
	%vars = ();
	foreach (split(/\n+/, $data))
	{
		$row++;
		
		next unless (/^<input.*name=\"?([^\" ]+)\"?.*value=\"?([^\" ]+)\"?/i);
		
		#print "$1 -> $2\n";
		$vars{$1} = $2;
	}
	
	# Populating SYSTEM with the vars
	#$SYSTEM->{'model'}      = ($vars{'productIdentificationDisplayName'}) ? "Dell $vars{'productIdentificationDisplayName'}" : '';
	
	$vars{'rndBasePhysicalAddress'} =~ /^(..)(..)(..)(..)(..)(..)$/;
	$SYSTEM->{'macaddress'} = uc "$1:$2:$3:$4:$5:$6";
	
	$SYSTEM->{'servicetag'} = $vars{'productIdentificationServiceTag$repeat?1'};
	$SYSTEM->{'assettag'}   = $vars{'productIdentificationAssetTag$repeat?1'};
	$SYSTEM->{'sysname'}    = $vars{'sysName'};
	
	$SYSTEM->{'location'}   = $vars{'sysLocation'};
	$SYSTEM->{'serial'}     = $vars{'productIdentificationSerialNumber$repeat?1'};
	
	# var sysUpTime="0005597773";
	# Convert into seconds
	$SYSTEM->{'old_uptime'} = $SYSTEM->{'uptime'} || 0;
	$SYSTEM->{'uptime'}     = int($vars{'sysUpTime'} / 100);	
	
	##
	## Fetching system info (versions)
	## Don't poll again unless switch rebooted
	##
	
	if (!$SYSTEM->{'firmware'} || $loop == 1 || ($SYSTEM->{'uptime'} < $SYSTEM->{'old_uptime'}))
	{
		$res = $ua->get("http://$SET->{'SWITCH'}/config/general_versions.htm",
		       'Cookie' => "$SYSTEM->{'cookie'}; checked=true;"
			 );
		
		&log_n_die("$logtag Unable to fetch /config/general_versions.htm - ".$res->status_line) unless ($res->is_success);
		
		# Parsing content
		$data = $res->content;
		$data =~ tr/\r//d;
		
		if ($SET->{'DEBUG'})
		{
			open (DUMP, ">$SYSTEM->{'BASEDIR'}/debug_config-general_versions.htm");
			print DUMP $data;
			close DUMP;
		}
		
		if ($data =~ /^<title>Authentication Page<\/title>$/m)
		{
			##
			## Invalid session!
			##
			
			&log("Session invalid", $SET->{'VERBOSE'});
			
			$SYSTEM->{'cookie'} = '';
			return 0;
		}
		
		# <INPUT  TYPE=hidden NAME="rndBrgVersion"   value="1.0.0.38" SIZE="20" MAXLENGTH="20">
		# <INPUT  TYPE=hidden NAME="rndBaseBootVersion"   value="1.0.0.12" SIZE="20" MAXLENGTH="20">
		# <INPUT  TYPE=hidden NAME="genGroupHWVersion"   value="00.00.02" SIZE="20" MAXLENGTH="20">
		
		$row = 0;
		%vars = ();
		foreach (split(/\n+/, $data))
		{
			$row++;
			
			next unless (/^<input.*name=\"?([^\" ]+)\"?.*value=\"?([^\" ]+)\"?/i);
			
			#print "$1 -> $2\n";
			$vars{$1} = $2;
		}
		
		$SYSTEM->{'firmware'}   = "$vars{'rndBrgVersion'}/$vars{'rndBaseBootVersion'} (hw $vars{'genGroupHWVersion'})";
	}
	
	##
	## Fetching model... Haven't seen it anywhere else than here (yet)
	## Don't poll again if we already got it, assume it doesn't change that often either :-)
	##
	
	if (!$SYSTEM->{'model'} || $loop == 1 || ($SYSTEM->{'uptime'} < $SYSTEM->{'old_uptime'}))
	{
		$res = $ua->get("http://$SET->{'SWITCH'}/device/logo.htm",
		       'Cookie' => "$SYSTEM->{'cookie'}; checked=true;"
			 );
		
		&log_n_die("$logtag Unable to fetch /device/logo.htm - ".$res->status_line) unless ($res->is_success);
		
		# Parsing content
		$data = $res->content;
		$data =~ tr/\r//d;
		
		if ($SET->{'DEBUG'})
		{
			open (DUMP, ">$SYSTEM->{'BASEDIR'}/debug_device-logo.htm");
			print DUMP $data;
			close DUMP;
		}
		
		if ($data =~ /^<title>Authentication Page<\/title>$/m)
		{
			##
			## Invalid session!
			##
			
			&log("Session invalid", $SET->{'VERBOSE'});
			
			$SYSTEM->{'cookie'} = '';
			return 0;
		}
		
		# <INPUT  type="hidden" id="productName" name="productIdentificationDisplayName"   value="PowerConnect&nbsp;2824" SIZE="20" MAXLENGTH="20">
		
		$row = 0;
		%vars = ();
		foreach (split(/\n+/, $data))
		{
			$row++;
			
			next unless (/^<input.*name=\"?([^\" ]+)\"?.*value=\"?([^\" ]+)\"?/i);
			
			#print "$1 -> $2\n";
			$vars{$1} = $2;
		}
		
		$SYSTEM->{'model'} = ($vars{'productIdentificationDisplayName'}) ? "Dell $vars{'productIdentificationDisplayName'}" : '';
	}
	
	# Fixing &nbsp;
	foreach (keys %$SYSTEM)
	{
		$SYSTEM->{$_} =~ s/&nbsp;/ /g;
	}
	
	return \%ports;
}

##
## Poll MACs
##

sub switch_poll_macs
{
	my $logtag = "$modname: poll_macs:";
	
	#?[ifTableVT]Filter:((ifIndex>=1)&&(ifIndex<=56))
	my $res = $ua->get("http://$SET->{'SWITCH'}/config/address_dynamic.htm",
	       'Cookie' => "$SYSTEM->{'cookie'}; checked=true;"
		 );
	
	&log_n_die("$logtag Unable to fetch /config/address_dynamic.htm - ".$res->status_line) unless ($res->is_success);
	
	# Content
	my $data = $res->content;
	$data =~ tr/\r//d;
	
	if ($SET->{'DEBUG'})
	{
		open (DUMP, ">$SYSTEM->{'BASEDIR'}/debug_config-address_dynamic.htm");
		print DUMP $data;
		close DUMP;
	}
	
	if ($data =~ /^<title>Authentication Page<\/title>$/m)
	{
		##
		## Invalid session!
		##
		
		&log("Session invalid", $SET->{'VERBOSE'});
		
		$SYSTEM->{'cookie'} = '';
		return 0;
	}
	
	##
	## Finds the info we needs
	##
	
	#<input name="dot1qTpFdbAddress$repeat?48" disabled="disabled" value="0019d182f993" size="20" maxlength="20" type="hidden">
	#<input name="dot1qFdbId$repeat?48" disabled="disabled" value="1" size="20" maxlength="20" type="hidden">
	#<input name="dot1qTpFdbPort$repeat?48" disabled="disabled" value="g1" size="20" maxlength="20" type="hidden">
	
	my $row = 0;
	my %vars = ();
	my $tot = 0;	#
	my $single_vlan = '';
	
	foreach (split(/\n+/, $data))
	{
		$row++;
		#print "RAD $row: $_\n";
		
		next unless (/^<input.*name=\"?([^\" ]+)\"?.*value=\"?([^\" ]+)\"?/i);
		
		my $name = $1;
		my $value = $2;
		
		#next if ($stat{$name});
		#print "  $name -> $value\n";
		$vars{$name} = $value;
		
		# Highest entry
		$tot = $1 if ($name =~ /^dot1qTpFdbAddress\$repeat\?(\d+)$/);
		
		if ($value =~ /^\d+$/ && $name =~ /^dot1qFdbId/)
		{
			$single_vlan ||= $value;
			$single_vlan = 'M' if ($single_vlan ne $value);
			#print "$value - $single_vlan\n";
		}
	}
	
	##
	## Summarizes mac -> note per port
	##
	
	my %port_macs = ();	# port -> hashref with mac -> note
	
	foreach my $num (1 .. $tot)
	{
		#<input name="dot1qTpFdbAddress$repeat?48" disabled="disabled" value="0019d182f993" size="20" maxlength="20" type="hidden">
		$vars{"dot1qTpFdbAddress\$repeat?$num"} =~ /^(..)(..)(..)(..)(..)(..)$/ || next;
		my $mac = uc "$1:$2:$3:$4:$5:$6";
		
		#<input name="dot1qTpFdbPort$repeat?48" disabled="disabled" value="g1" size="20" maxlength="20" type="hidden">
		my $ifName = $vars{"dot1qTpFdbPort\$repeat?$num"};
		
		my $portnum = '';
		if ($ifName =~ /^g(\d+)$/)
		{
			# Normal gigport
			$portnum = $1;
		}
		elsif ($ifName =~ /^ch(\d+)$/)
		{
			# LAG, I keep the T...
			$portnum = "T$1";
		}
		else
		{
			next;
		}
		
		#<input name="dot1qFdbId$repeat?48" disabled="disabled" value="1" size="20" maxlength="20" type="hidden">
		my $vlan   = $vars{"dot1qFdbId\$repeat?$num"};
		
		my $macs = $port_macs{$portnum} ||= {};
		$macs->{$mac} = ($single_vlan ne 'M' && $vlan) ? '' : "VLAN $vlan";
	}
	
	return \%port_macs;
}

##
## Login to setup the session
##

sub switch_login
{
	my $logtag = "$modname: login:";
	
	my $sock = Net::HTTP->new(Host => $SET->{'SWITCH'}) || die $@;
	$sock->write_request(POST => "/config/log_off_page.htm",
					'User-Agent' => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)",
					'Content-Type' => 'application/x-www-form-urlencoded',
					'Cache-Control' => 'no-cache',
					'Accept' => '*/*',
					"restoreUrl=&errorCollector=&userName%24query=$SET->{'USERNAME'}&password%24query=$SET->{'PASSWORD'}"
			);
	
	my($code, $mess, %h) = $sock->read_response_headers;
	
	unless ($code == 302)
	{
		&log_n_die("$logtag Didn't recieve code 302 when logging in! $mess");
	}
	
	##
	## Finding the Cookie we should recieve when logged in successfully
	##
	## Set-Cookie : UserId=212.116.70.62&1120127020&; path=/
	##
	
	unless ($h{'Set-Cookie'})
	{
		foreach (sort keys %h)
		{
			print "$_ = $h{$_}\n";
		}
			
		&log_n_die("$logtag Didn't recieve cookie! $h{'Set-Cookie'}");
	}
	
	$h{'Set-Cookie'} =~ /(UserId=.+?)\;/ || &log_n_die("$logtag Unable to interpret UserID from $h{'Set-Cookie'}!");
	return $1;
}

##
## Logout - Unsure if this is the correct way
## The 2748 is very robust, never hanged the webif yet while working it
##

sub switch_logout
{
	my $logtag = "$modname: logout";
	
	# GET /config/authentication_page.htm?logOff HTTP/1.1
	my $res = $ua->get("http://$SET->{'SWITCH'}/config/authentication_page.htm?logOff",
	       'Cookie' => "$SYSTEM->{'cookie'}; checked=false"
		 );
	
	&log_n_die("$logtag: Unable to fetch /config/authentication_page.htm?logOff - ".$res->status_line) unless ($res->is_success);
	my $data = $res->content;
	
	#print $data;
	
	# Assume it worked
	$SYSTEM->{'cookie'} = '';
	
	return 1;
}

###########################################################

&log("$modname $modver loaded", $SET->{'VERBOSE'});

1;

